@@ -18,4 +18,4 @@ CORBA_ORB		    bonobo_orb			 (void);
 PortableServer_POA	    bonobo_poa			 (void);
 PortableServer_POAManager  bonobo_poa_manager		 (void);
 
-#endif  __GNOME_MAIN_H__
+#endif /* __GNOME_MAIN_H__ */
