@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /**
- * GNOME Unknown interface base implementation
+ * Bonobo Unknown interface base implementation
  *
  * Author:
  *   Miguel de Icaza (miguel@kernel.org)
@@ -73,7 +73,6 @@ bonobo_object_from_servant (PortableServer_Servant servant)
 PortableServer_Servant
 bonobo_object_get_servant (BonoboObject *object)
 {
-	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), NULL);
 
 	return object->servant;
@@ -94,9 +93,8 @@ bonobo_object_get_servant (BonoboObject *object)
 void
 bonobo_object_bind_to_servant (BonoboObject *object, void *servant)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (servant != NULL);
 	g_return_if_fail (BONOBO_IS_OBJECT (object));
+	g_return_if_fail (servant != NULL);
 
 	object->servant = servant;
 	((BonoboObjectServant *)servant)->bonobo_object = object;
@@ -111,7 +109,6 @@ bonobo_object_bind_to_servant (BonoboObject *object, void *servant)
 void
 bonobo_object_ref (BonoboObject *object)
 {
-	g_return_if_fail (object != NULL);
 	g_return_if_fail (BONOBO_IS_OBJECT (object));
 	g_return_if_fail (object->priv->ao->ref_count > 0);
 
@@ -127,25 +124,13 @@ bonobo_object_ref (BonoboObject *object)
 void
 bonobo_object_unref (BonoboObject *object)
 {
-	g_return_if_fail (object != NULL);
 	g_return_if_fail (BONOBO_IS_OBJECT (object));
 	g_return_if_fail (object->priv->ao->ref_count > 0);
 
-	object->priv->ao->ref_count--;
-
-	if (object->priv->ao->ref_count == 0){
-		GnomeAggregateObject *ao = object->priv->ao;
-		GList *l;
-
-		for (l = ao->objs; l; l = l->next){
-			BonoboObject *o = l->data;
-
-			gtk_signal_disconnect (GTK_OBJECT (o), o->priv->destroy_id);
-			gtk_object_destroy (GTK_OBJECT (o));
-		}
-
-		g_list_free (ao->objs);
-		g_free (ao);
+	if (object->priv->ao->ref_count == 1) {
+		bonobo_object_destroy (object);
+	} else {
+		object->priv->ao->ref_count--;
 	}
 }
 
@@ -153,6 +138,10 @@ bonobo_object_unref (BonoboObject *object)
  * bonobo_object_destroy
  * @object: A BonoboObject you want to destroy.
  *
+ * This function is deprecated. Use bonobo_object_unref instead.
+ * This is *not* like gtk_object_destroy, which can be done independently
+ * of gtk_object_unref and has no effect on the ref count.
+ *
  * Destroys an object.  Ignores all reference count.  Used when you want
  * to force the destruction of the composite object (use only if you know
  * what you are doing).
@@ -163,12 +152,12 @@ bonobo_object_destroy (BonoboObject *object)
 	GnomeAggregateObject *ao;
 	GList *l;
 
-	g_return_if_fail (object != NULL);
 	g_return_if_fail (BONOBO_IS_OBJECT (object));
 	g_return_if_fail (object->priv->ao->ref_count > 0);
 
 	ao = object->priv->ao;
 
+	ao->ref_count = 0;
 	for (l = ao->objs; l; l = l->next){
 		BonoboObject *o = l->data;
 
@@ -215,7 +204,6 @@ bonobo_object_get_local_interface_from_objref (BonoboObject *object,
 
 		if (CORBA_Object_is_equivalent (interface, tryme->corba_objref, &ev)) {
 			CORBA_exception_free (&ev);
-
 			return tryme;
 		}
 
@@ -248,7 +236,6 @@ bonobo_object_query_local_interface (BonoboObject *object,
 	GtkType            type;
 	GList             *l;
 
-	g_return_val_if_fail (object != NULL,            NULL);
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), NULL);
 
 	retval       = NULL;
@@ -406,7 +393,7 @@ bonobo_object_class_init (BonoboObjectClass *klass)
 static void
 bonobo_object_usage_error (BonoboObject *object)
 {
-	g_error ("Aggregate bonobo_object member %p has been destroyed using gtk_object_* methods\n", object);
+	g_error ("Aggregate bonobo_object member %p has been destroyed using gtk_object_* methods", object);
 }
 
 static void
@@ -469,7 +456,6 @@ bonobo_object_activate_servant (BonoboObject *object, void *servant)
 	CORBA_Environment ev;
 	CORBA_Object o;
 
-	g_return_val_if_fail (object != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), CORBA_OBJECT_NIL);
 	g_return_val_if_fail (servant != NULL, CORBA_OBJECT_NIL);
 
@@ -499,14 +485,13 @@ bonobo_object_activate_servant (BonoboObject *object, void *servant)
  *
  * Initializes the provided BonoboObject @object. This method is
  * usually invoked from the construct method for other Gtk-based CORBA
- * wrappers that derive from the GNOME::obj interface
+ * wrappers that derive from the Bonobo::Unknown interface
  *
  * Returns: the initialized BonoboObject.
  */
 BonoboObject *
 bonobo_object_construct (BonoboObject *object, CORBA_Object corba_object)
 {
-	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (corba_object != CORBA_OBJECT_NIL, NULL);
 
@@ -536,6 +521,9 @@ bonobo_object_add_interface (BonoboObject *object, BonoboObject *newobj)
        if (object->priv->ao == newobj->priv->ao)
                return;
 
+       if (newobj->corba_objref == CORBA_OBJECT_NIL)
+	       g_warning ("Adding an interface with a NULL Corba objref");
+
        /*
 	* Explanation:
 	*   Bonobo Objects should not be assembled after they have been
@@ -574,9 +562,10 @@ bonobo_object_query_interface (BonoboObject *object, const char *repo_id)
 
        CORBA_exception_init(&ev);
        retval = Bonobo_Unknown_query_interface (object->corba_objref, (CORBA_char *)repo_id, &ev);
-       if(ev._major != CORBA_NO_EXCEPTION)
+       if (ev._major != CORBA_NO_EXCEPTION)
                retval = CORBA_OBJECT_NIL;
        CORBA_exception_free (&ev);
+
        return retval;
 }
 
@@ -589,7 +578,6 @@ bonobo_object_query_interface (BonoboObject *object, const char *repo_id)
 CORBA_Object
 bonobo_object_corba_objref (BonoboObject *object)
 {
-	g_return_val_if_fail (object != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), NULL);
 
 	return object->corba_objref;
@@ -610,9 +598,8 @@ bonobo_object_corba_objref (BonoboObject *object)
 void
 bonobo_object_check_env (BonoboObject *object, CORBA_Object obj, CORBA_Environment *ev)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (ev != NULL);
 	g_return_if_fail (BONOBO_IS_OBJECT (object));
+	g_return_if_fail (ev != NULL);
 
 	if (ev->_major == CORBA_NO_EXCEPTION)
 		return;
@@ -630,7 +617,7 @@ bonobo_object_check_env (BonoboObject *object, CORBA_Object obj, CORBA_Environme
  * Pings the object @object using the ref/unref methods from Bonobo::Unknown.
  * You can use this one to see if a remote object has gone away.
  *
- * Returns: %TRUE if the GNOME::Unkonwn @object is alive.
+ * Returns: %TRUE if the Bonobo::Unknown @object is alive.
  */
 gboolean
 gnome_unknown_ping (Bonobo_Unknown object)
