@@ -96,8 +96,7 @@ typedef enum {
 	BONOBO_UI_HANDLER_PIXMAP_STOCK,
 	BONOBO_UI_HANDLER_PIXMAP_FILENAME,
 	BONOBO_UI_HANDLER_PIXMAP_XPM_DATA,
-	BONOBO_UI_HANDLER_PIXMAP_RGBA_DATA,
-	BONOBO_UI_HANDLER_PIXMAP_RGB_DATA
+	BONOBO_UI_HANDLER_PIXMAP_PIXBUF_DATA
 } BonoboUIHandlerPixmapType;
 
 typedef enum {
