@@ -7,6 +7,7 @@
  */
 #define BONOBO_UI_HANDLER_COMPILATION
 
+#include <config.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenu.h>
@@ -273,7 +274,7 @@ toolbar_type_to_corba (BonoboUIHandlerToolbarItemType type)
 {
 	switch (type) {
 	case BONOBO_UI_HANDLER_TOOLBAR_END:
-		g_warning ("Passing ToolbarTypeEnd through CORBA!\n");
+		g_warning ("Passing ToolbarTypeEnd through CORBA!");
 		return Bonobo_UIHandler_ToolbarTypeEnd;
 
 	case BONOBO_UI_HANDLER_TOOLBAR_ITEM:
@@ -295,7 +296,7 @@ toolbar_type_to_corba (BonoboUIHandlerToolbarItemType type)
 		return Bonobo_UIHandler_ToolbarTypeControl;
 
 	default:
-		g_warning ("toolbar_type_to_corba: Unknown toolbar type [%d]!\n", (gint) type);
+		g_warning ("toolbar_type_to_corba: Unknown toolbar type [%d]!", (gint) type);
 	}
 
 	return BONOBO_UI_HANDLER_TOOLBAR_ITEM;
@@ -306,7 +307,7 @@ toolbar_corba_to_type (Bonobo_UIHandler_ToolbarType type)
 {
 	switch (type) {
 	case Bonobo_UIHandler_ToolbarTypeEnd:
-		g_warning ("Passing ToolbarTypeEnd through CORBA!\n");
+		g_warning ("Passing ToolbarTypeEnd through CORBA!");
 		return BONOBO_UI_HANDLER_TOOLBAR_END;
 
 	case Bonobo_UIHandler_ToolbarTypeItem:
@@ -328,7 +329,7 @@ toolbar_corba_to_type (Bonobo_UIHandler_ToolbarType type)
 		return BONOBO_UI_HANDLER_TOOLBAR_CONTROL;
 
 	default:
-		g_warning ("toolbar_corba_to_type: Unknown toolbar type [%d]!\n", (gint) type);
+		g_warning ("toolbar_corba_to_type: Unknown toolbar type [%d]!", (gint) type);
 			
 	}
 
@@ -943,7 +944,7 @@ toolbar_local_remove_parent_entry (BonoboUIHandler *uih, const char *path,
 	}
 
 	if (warn)
-		g_warning ("toolbar_local_remove_parent_entry: No entry in toolbar for item path [%s]!\n", path);
+		g_warning ("toolbar_local_remove_parent_entry: No entry in toolbar for item path [%s]!", path);
 }
 
 static void
@@ -1016,7 +1017,7 @@ impl_Bonobo_UIHandler_toolbar_overridden (PortableServer_Servant  servant,
 	internal_cb = toolbar_local_get_item (uih, path);
 
 	if (internal_cb == NULL && strcmp (parent_path, "/")) {
-		g_warning ("Received override notification for a toolbar item I don't own [%s]!\n", path);
+		g_warning ("Received override notification for a toolbar item I don't own [%s]!", path);
 		return;
 	}
 
@@ -1069,7 +1070,7 @@ impl_Bonobo_UIHandler_toolbar_activated (PortableServer_Servant  servant,
 	internal_cb = toolbar_local_get_item (uih, path);
 
 	if (internal_cb == NULL) {
-		g_warning ("Received activation notification about a toolbar item I don't own [%s]!\n", path);
+		g_warning ("Received activation notification about a toolbar item I don't own [%s]!", path);
 		return;
 	}
 
@@ -1097,12 +1098,12 @@ toolbar_type_to_gtk_type (BonoboUIHandlerToolbarItemType type)
 
 	case BONOBO_UI_HANDLER_TOOLBAR_RADIOGROUP:
 		g_warning ("toolbar_type_to_gtk_type: Trying to convert UIHandler radiogroup "
-			   "to Gtk toolbar type!\n");
+			   "to Gtk toolbar type!");
 		return GTK_TOOLBAR_CHILD_SPACE;
 		
 	default:
 		g_warning ("toolbar_type_to_gtk_type: Unkonwn UIHandler toolbar "
-			   "item type [%d]!\n", type);
+			   "item type [%d]!", type);
 	}
 
 	return GTK_TOOLBAR_CHILD_BUTTON;
@@ -1200,12 +1201,11 @@ toolbar_toplevel_item_create_widgets (BonoboUIHandler *uih,
 		break;
 
 	case BONOBO_UI_HANDLER_TOOLBAR_RADIOITEM:
-		g_warning ("Toolbar radio items/groups not implemented\n");
+		g_warning ("Toolbar radio items/groups not implemented");
 		return;
 		
 	default:
-
-		g_warning ("toolbar_toplevel_item_create_widgets: Unkonwn toolbar item type [%d]!\n",
+		g_warning ("toolbar_toplevel_item_create_widgets: Unkonwn toolbar item type [%d]!",
 			   (gint) internal->item->type);
 		return;
 	}
@@ -1234,7 +1234,7 @@ toolbar_toplevel_item_remove_parent_entry (BonoboUIHandler *uih, const char *pat
 	parent = toolbar_toplevel_get_toolbar (uih, parent_name);
 
 	if (parent == NULL) {
-		g_warning ("toolbar_toplevel_remove_parent_entry: Cannot find toolbar [%s] for path [%s]!\n",
+		g_warning ("toolbar_toplevel_remove_parent_entry: Cannot find toolbar [%s] for path [%s]!",
 			   path, parent_name);
 		g_free (parent_name);
 		return;
@@ -1251,7 +1251,7 @@ toolbar_toplevel_item_remove_parent_entry (BonoboUIHandler *uih, const char *pat
 	}
 
 	if (warn)
-		g_warning ("toolbar_toplevel_remove_parent_entry: No entry in parent for child path [%s]!\n", path);
+		g_warning ("toolbar_toplevel_remove_parent_entry: No entry in parent for child path [%s]!", path);
 }
 
 static void
@@ -1797,7 +1797,7 @@ impl_Bonobo_UIHandler_toolbar_reinstated (PortableServer_Servant  servant,
 	internal_cb = toolbar_local_get_item (uih, path);
 
 	if (internal_cb == NULL) {
-		g_warning ("Received reinstatement notification for toolbar item I don't own [%s]!\n", path);
+		g_warning ("Received reinstatement notification for toolbar item I don't own [%s]!", path);
 		return;
 	}
 
@@ -1830,7 +1830,7 @@ impl_Bonobo_UIHandler_toolbar_removed (PortableServer_Servant  servant,
 	internal_cb = toolbar_local_get_item (uih, path);
 
 	if (internal_cb == NULL) {
-		g_warning ("Received removal notification for toolbar item I don't own [%s]!\n", path);
+		g_warning ("Received removal notification for toolbar item I don't own [%s]!", path);
 		return;
 	}
 
@@ -1883,13 +1883,6 @@ toolbar_toplevel_item_remove_data (BonoboUIHandler *uih, ToolbarItemInternal *in
 
 	toolbar_free (internal->item);
 
-	/*
-	 * FIXME
-	 *
-	 * This g_slist_free() (or maybe the other one in this file)
-	 * seems to corrupt the SList allocator's free list
-	 */
-/*	g_slist_free (internal->radio_items); */
 	g_free (internal);
 	g_free (path);
 }
@@ -2067,23 +2060,23 @@ toolbar_uiinfo_type_to_uih (GnomeUIInfoType uii_type)
 		return BONOBO_UI_HANDLER_TOOLBAR_RADIOGROUP;
 
 	case GNOME_APP_UI_SUBTREE:
-		g_error ("Toolbar subtree doesn't make sense!\n");
+		g_error ("Toolbar subtree doesn't make sense!");
 
 	case GNOME_APP_UI_SEPARATOR:
 		return BONOBO_UI_HANDLER_TOOLBAR_SEPARATOR;
 
 	case GNOME_APP_UI_HELP:
-		g_error ("Help unimplemented.\n"); /* FIXME */
+		g_error ("Help unimplemented."); /* FIXME */
 
 	case GNOME_APP_UI_BUILDER_DATA:
-		g_error ("Builder data - what to do?\n"); /* FIXME */
+		g_error ("Builder data - what to do?"); /* FIXME */
 
 	case GNOME_APP_UI_ITEM_CONFIGURABLE:
 		g_warning ("Configurable item!");
 		return BONOBO_UI_HANDLER_MENU_ITEM;
 
 	case GNOME_APP_UI_SUBTREE_STOCK:
-		g_error ("Toolbar subtree doesn't make sense!\n");
+		g_error ("Toolbar subtree doesn't make sense!");
 
 	default:
 		g_warning ("Unknown UIInfo Type: %d", uii_type);
