@@ -891,6 +891,47 @@ static void gtk_minefield_cleanup (GtkMineFieldView *mfield)
         mfield->multi_mode = 0;
 }
 
+void minefield_set_at (MineField *data, int x, int y,
+		       char num, char state)
+{
+	int idx;
+
+	g_return_if_fail (data != NULL);
+
+	idx = x + y * data->xsize;
+	g_return_if_fail (idx < data->xsize * data->ysize);
+
+	data->mines [idx].neighbours = num - '0';
+	data->mines [idx].mined      = state == 'M';
+	data->mines [idx].marked     = state == 'P';
+	data->mines [idx].down       = state == 'v';
+	data->mines [idx].shown      = state == ' ';
+}
+
+void minefield_get_at (MineField *data, int x, int y,
+		       char *num, char *state)
+{
+	int idx;
+
+	g_return_if_fail (data != NULL);
+
+	idx = x + y * data->xsize;
+	g_return_if_fail (idx < data->xsize * data->ysize);
+
+	*num = data->mines [idx].neighbours + '0';
+
+	if (data->mines [idx].mined)
+		*state = 'M';
+	else if (data->mines [idx].marked)
+		*state = 'P';
+	else if (data->mines [idx].down)
+		*state = 'v';
+	else if (data->mines [idx].shown)
+		*state = ' ';
+	else
+		*state = '?';
+}
+
 void minefield_restart (MineField *data)
 {
 	guint i, j;
