@@ -415,9 +415,15 @@ embeddable_factory (BonoboEmbeddableFactory *this,  void *data)
 static void
 init_bonobo_audio_ulaw_factory (void)
 {
+#if USING_OAF
+	factory = bonobo_embeddable_factory_new (
+	 "OAFIID:bonobo_audio-ulaw_factory:823935e2-c944-42e7-a462-75cc76b18abb",
+		 embeddable_factory, NULL);
+#else
 	factory = bonobo_embeddable_factory_new (
 		"embeddable-factory:audio-ulaw",
 		 embeddable_factory, NULL);
+#endif
 } /* init_bonobo_audio_ulaw_factory */
 
 static void
@@ -428,13 +434,21 @@ init_server_factory (int argc, char **argv)
 
 	CORBA_exception_init (&ev);
 
+#if USING_OAF
+        gnome_init_with_popt_table("bonobo-audio-ulaw", VERSION,
+				   argc, argv,
+				   oaf_popt_options, 0, NULL); 
+	orb =oaf_init (argc, argv);
+#else
 	gnome_CORBA_init_with_popt_table (
 		"bonobo-audio-ulaw", VERSION,
 		&argc, argv, NULL, 0, NULL, GNORBA_INIT_SERVER_FUNC, &ev);
 
+	orb = gnome_CORBA_ORB ();
+#endif
+
 	color_init();
 
-	orb = gnome_CORBA_ORB ();
 	if (bonobo_init (orb, NULL, NULL) == FALSE)
 		g_error (_("I could not initialize Bonobo"));
 
